package cn.com.qmth.stmms.ms.admin.api;

import cn.com.qmth.stmms.ms.commons.utils.ServletUtil;
import cn.com.qmth.stmms.ms.core.domain.MarkStage;
import cn.com.qmth.stmms.ms.core.domain.Message;
import cn.com.qmth.stmms.ms.core.domain.user.MarkUser;
import cn.com.qmth.stmms.ms.core.repository.MessageRepo;
import cn.com.qmth.stmms.ms.core.vo.Subject;
import com.alibaba.fastjson.JSONObject;
import org.springframework.data.domain.Page;
import org.springframework.data.domain.PageRequest;
import org.springframework.data.domain.Pageable;
import org.springframework.data.domain.Sort;
import org.springframework.data.jpa.domain.Specification;
import org.springframework.web.bind.annotation.*;

import javax.annotation.Resource;
import javax.persistence.criteria.Predicate;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;

/**
 * 消息管理
 */
@RestController
@RequestMapping("api/message")
public class MessageApi {

    @Resource
    private MessageRepo messageRepo;

    @PostMapping("/save")
    public void save(@RequestBody JSONObject data) {
        Long workId = ServletUtil.getWordId();
        List<MarkUser> markUsers = JSONObject.parseArray(data.getString("markers"), MarkUser.class);
        List<Message> messages = new ArrayList<>();
        long time = System.currentTimeMillis();
        for (MarkUser markUser : markUsers) {
            Message message = new Message();
            message.setWorkId(workId);
            message.setSubject(Subject.valueOf(data.getString("subject")));
            message.setMarkerId(markUser.getId());
            message.setMarkerName(markUser.getName());
            message.setMarkerLeaderId(data.getLong("markerLeaderId"));
            message.setMarkerLeaderName(data.getString("markerLeaderName"));
            message.setContent(data.getString("content"));
            message.setCreateTime(time);
            message.setRead(false);
            messages.add(message);
        }
        messageRepo.save(messages);
    }

    /**
     * 评卷员查询消息
     *
     * @param subject  科目
     * @param stage    阶段
     * @param markerId 评卷员ID
     * @param lastId   上次最后一次查询的消息ID最大值
     * @param pageable 分页
     */
    @GetMapping("/list_by_marker")
    public Page<Message> listByMarker(@RequestParam Subject subject,
                                      @RequestParam MarkStage stage,
                                      @RequestParam Long markerId,
                                      @RequestParam Long lastId,
                                      Pageable pageable) {
        Long workId = ServletUtil.getWordId();
        Specification<Message> specification = (root, query, builder) -> {
            List<Predicate> predicates = new ArrayList<>();
            if (lastId != null) {
                predicates.add(builder.gt(root.get("id"), lastId));
            }
            predicates.add(builder.equal(root.get("workId"), workId));
            predicates.add(builder.equal(root.get("subject"), subject));
            predicates.add(builder.equal(root.get("stage"), stage));
            predicates.add(builder.equal(root.get("markerId"), markerId));
            predicates.add(builder.equal(root.get("read"), false));

            return builder.and(predicates.toArray(new Predicate[predicates.size()]));
        };

        Sort sort = new Sort(Sort.Direction.DESC, "id");
        Pageable pageable1 = new PageRequest(pageable.getPageNumber(), pageable.getPageSize(), sort);
        return messageRepo.findAll(specification, pageable1);
    }

    @GetMapping("/list_by_marker_leader")
    public void listByMarkerLeader(@RequestParam Subject subject,
                                   @RequestParam MarkStage stage,
                                   @RequestParam Long markerLeaderId) {
    }

    /**
     * 标记已读
     *
     * @param id 消息ID
     */
    @GetMapping("/read/{id}")
    public void listByMarkerLeader(@PathVariable Long id) {
        Message message = messageRepo.findOne(id);
        message.setRead(true);
        messageRepo.save(message);
    }

}
